Package: daltoolboxdp
Title: Deep Python Extensions for 'daltoolbox'
Version: 1.3.747
Authors@R: 
  c(    person(given = "Eduardo", family = "Ogasawara", role = c("aut", "ths", "cre"), 
               email = "eogasawara@ieee.org", comment = c(ORCID = "0000-0002-0466-0626")),
        person(given = "Diego", family = "Salles", role = c("aut"), email = "diego.salles@eic.cefet-rj.br"),
        person(given = "Erich", family = "Carvalho", role = c("aut"), email = "erich.carvalho@eic.cefet-rj.br"),
        person(given = "Janio", family = "Lima", role = c("aut"), email = "janio.lima@eic.cefet-rj.br"),
        person(given = "Joao", family = "Kongevold", role = c("aut"), email = "joao.kongevold@eic.cefet-rj.br"),
        person(given = "Lucas", family = "Tavares", role = c("aut"), email = "lucas.tavares@eic.cefet-rj.br"),
        person(given = "Eduardo", family = "Bezerra", role = c("ctb"), email = "ebezerra@cefet-rj.br"),
        person(given = "CEFET/RJ", role = "cph")        
  )
Description: 
  Extends 'daltoolbox' with Python-backed components for deep learning, 
  scikit-learn classification, and time-series forecasting through 
  'reticulate'. The package provides objects that follow the 'daltoolbox' 
  architecture while delegating model creation, fitting, encoding, and 
  prediction to Python libraries such as 'torch' and 'scikit-learn'. In the 
  package name, 'dp' stands for 'Deep Python'. The overall workflow is 
  inspired by the Experiment Lines approach described in Ogasawara et al. 
  (2009) <doi:10.1007/978-3-642-02279-1_20>.
License: MIT + file LICENSE
URL: https://cefet-rj-dal.github.io/daltoolboxdp/,
        https://github.com/cefet-rj-dal/daltoolboxdp
BugReports: https://github.com/cefet-rj-dal/daltoolboxdp/issues
Encoding: UTF-8
RoxygenNote: 8.0.0
Depends: R (>= 4.1.0)
Imports: tspredit, daltoolbox, reticulate
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
Config/reticulate: list( packages = list( list(package = "scipy"),
        list(package = "torch"), list(package = "pandas"), list(package
        = "numpy"), list(package = "matplotlib"), list(package =
        "scikit-learn") ) )
NeedsCompilation: no
Packaged: 2026-05-22 12:01:59 UTC; gpca
Author: Eduardo Ogasawara [aut, ths, cre] (ORCID:
    <https://orcid.org/0000-0002-0466-0626>),
  Diego Salles [aut],
  Erich Carvalho [aut],
  Janio Lima [aut],
  Joao Kongevold [aut],
  Lucas Tavares [aut],
  Eduardo Bezerra [ctb],
  CEFET/RJ [cph]
Maintainer: Eduardo Ogasawara <eogasawara@ieee.org>
Repository: CRAN
Date/Publication: 2026-05-22 12:40:09 UTC
Built: R 4.6.0; ; 2026-05-22 17:30:43 UTC; unix
