# Generated by roxygen2: do not edit by hand

S3method(do_fit,torch_ts_mlp)
S3method(do_fit,ts_conv1d)
S3method(do_fit,ts_lstm)
S3method(do_predict,torch_ts_mlp)
S3method(do_predict,ts_conv1d)
S3method(do_predict,ts_lstm)
S3method(fit,autoenc_adv_e)
S3method(fit,autoenc_adv_ed)
S3method(fit,autoenc_conv_e)
S3method(fit,autoenc_conv_ed)
S3method(fit,autoenc_denoise_e)
S3method(fit,autoenc_denoise_ed)
S3method(fit,autoenc_e)
S3method(fit,autoenc_ed)
S3method(fit,autoenc_lstm_e)
S3method(fit,autoenc_lstm_ed)
S3method(fit,autoenc_stacked_e)
S3method(fit,autoenc_stacked_ed)
S3method(fit,autoenc_variational_e)
S3method(fit,autoenc_variational_ed)
S3method(fit,skcla_gb)
S3method(fit,skcla_knn)
S3method(fit,skcla_mlp)
S3method(fit,skcla_nb)
S3method(fit,skcla_rf)
S3method(fit,skcla_svc)
S3method(fit,torch_cla_mlp)
S3method(fit,torch_reg_mlp)
S3method(predict,skcla_gb)
S3method(predict,skcla_knn)
S3method(predict,skcla_mlp)
S3method(predict,skcla_nb)
S3method(predict,skcla_rf)
S3method(predict,skcla_svc)
S3method(predict,torch_cla_mlp)
S3method(predict,torch_reg_mlp)
S3method(transform,autoenc_adv_e)
S3method(transform,autoenc_adv_ed)
S3method(transform,autoenc_conv_e)
S3method(transform,autoenc_conv_ed)
S3method(transform,autoenc_denoise_e)
S3method(transform,autoenc_denoise_ed)
S3method(transform,autoenc_e)
S3method(transform,autoenc_ed)
S3method(transform,autoenc_lstm_e)
S3method(transform,autoenc_lstm_ed)
S3method(transform,autoenc_stacked_e)
S3method(transform,autoenc_stacked_ed)
S3method(transform,autoenc_variational_e)
S3method(transform,autoenc_variational_ed)
export(autoenc_adv_e)
export(autoenc_adv_ed)
export(autoenc_conv_e)
export(autoenc_conv_ed)
export(autoenc_denoise_e)
export(autoenc_denoise_ed)
export(autoenc_e)
export(autoenc_ed)
export(autoenc_lstm_e)
export(autoenc_lstm_ed)
export(autoenc_stacked_e)
export(autoenc_stacked_ed)
export(autoenc_variational_e)
export(autoenc_variational_ed)
export(skcla_gb)
export(skcla_knn)
export(skcla_mlp)
export(skcla_nb)
export(skcla_rf)
export(skcla_svc)
export(torch_cla_mlp)
export(torch_reg_mlp)
export(torch_ts_mlp)
export(ts_conv1d)
export(ts_lstm)
import(daltoolbox)
import(reticulate)
importFrom(daltoolbox,autoenc_base_e)
importFrom(daltoolbox,autoenc_base_ed)
importFrom(tspredit,do_fit)
importFrom(tspredit,do_predict)
importFrom(tspredit,ts_regsw)
